.field-certificate p,
.field-certificate .vLargeTextField,
.field-private_key p,
.field-private_key .vLargeTextField,
pre.x509{
    font-family: monospace;
    white-space: pre;
}
.field-certificate .vLargeTextField,
.field-private_key .vLargeTextField{
    height: 350px;
}
.field-certificate p.help,
.field-private_key p.help{
    font-family: inherit;
    white-space: normal;
}
fieldset.x509-custom { margin-top: 0 }
fieldset.x509-custom .form-row{ padding: 0 }
fieldset.x509-custom pre{ margin-top: 0 }
